@@ -1,5 +1,5 @@
 /*
- *  $Id: cachesize.c,v 1.13 2004/12/22 22:34:08 davej Exp $
+ *  $Id: cachesize.c,v 1.14 2005/08/07 18:42:37 davej Exp $
  *  This file is part of x86info.
  *  (C) 2001 Dave Jones.
  *
@@ -108,6 +108,9 @@ static struct _cache_table DTLB_cache_table[] =
 	{ 0, 0, 0 }
 };
 
+static char found_unknown=0;
+static char unknown_array[256];
+
 /* Decode Intel TLB and cache info descriptors */
 //TODO : Errata workaround. http://www.sandpile.org/post/msgs/20002736.htm
 static void decode_Intel_cache (int des, struct cpudata *cpu, int output,
@@ -141,6 +144,10 @@ static void decode_Intel_cache (int des, struct cpudata *cpu, int output,
 		}
 		k++;
 	}
+	if (table[k].descriptor == 0) {
+		unknown_array[des]=1;
+		found_unknown=1;
+	}
 }
 
 static void decode_cache(struct cpudata *cpu, struct _cache_table *table, int output)
@@ -173,15 +180,30 @@ static void decode_cache(struct cpudata *cpu, struct _cache_table *table, int ou
 
 void decode_Intel_caches (struct cpudata *cpu, int output)
 {
+	int i;
+
 	if (cpu->maxi < 2)
 		return;
 
+	memset (&unknown_array, 0, sizeof(unknown_array));
+
 	decode_cache (cpu, TRACE_cache_table, output);
 	decode_cache (cpu, L1I_cache_table, output);
 	decode_cache (cpu, L1D_cache_table, output);
 	decode_cache (cpu, L2_cache_table, output);
 	decode_cache (cpu, L3_cache_table, output);
 	decode_cache (cpu, ITLB_cache_table, output);
+
+	if (found_unknown == 0)
+		return;
+
+	printf ("Found unknown cache descriptors: ");
+
+	for (i=0; i<256; i++) {
+		if (unknown_array[i]==1)
+			printf ("%d ", i);
+	}
+	printf ("\n");
 	decode_cache (cpu, DTLB_cache_table, output);
 }
 
