@@ -526,6 +526,12 @@ void Identify_Intel (struct cpudata *cpu)
 		nameptr += sprintf (cpu->name, "%s", "Pentium M ");
 		cpu->connector = CONN_MICROFCBGA;
 		switch (cpu->stepping) {
+			case 1:	nameptr+=sprintf (cpu->name, "%s", "(Dothan) [A-1]");
+					break;
+
+			case 2:	nameptr+=sprintf (cpu->name, "%s", "(Dothan) [A-2]");
+					break;
+
 			/*
 				S-spec	Processor	Hi-Freq	Low-Freq
 						Number
@@ -540,10 +546,51 @@ void Identify_Intel (struct cpudata *cpu)
 				SL7GL	715			1.5GHz	600MHz
 				SL7GK	715			1.5GHz	600MHz
 				SL7VC	738			1.4GHz	600MHz
+				SL7F4	733			1.1GHz	600MHz
 				SL7VD	733			1.1GHz	600MHz
 				SL7V2	723			1.0GHz	600MHz
 			 */
-			case 6:	nameptr+=sprintf (cpu->name, "%s", "[B-1]");
+			case 6:	nameptr+=sprintf (cpu->name, "%s", "(Dothan) [B-1]");
+					break;
+
+			/*
+			   FSB=533 2MB L2 90nm
+				SL86G	730			1.6GHz	800MHz
+				SL7SA	740			1.73GHz	800MHz
+				SL7S9	750			1.86GHz	800MHz
+				SL7SM	760			2.0GHz	800MHz
+				SL7SL	770			2.13GHz	800MHz
+				SL7VB	780			2.26GHz	800MHz
+				SL86M	730			1.6GHz	800MHz
+				SL7S8	740			1.73GHz	800MHz
+				SL7SR	750			1.86GHz	800MHz
+				Sl7SQ	760			2.0GHz	800MHz
+				SL7SP	770			2.13GHz	800MHz
+				SL7SN	780			2.26GHz	800MHz
+				SL86B	740			1.73GHz	800MHz
+				SL86A	750			1.86GHz	800MHz
+				SL869	760			2.0GHz	800MHz
+				SL868	770			2.13GHz	800MHz
+				SL8QK	780			2.26GHz	800MHz
+			   FSB=400
+				SL8QF	778			1.6GHz	600MHz
+				SL89X	758			1.5GHz	600MHz
+				SL8A3	723			1.0GHz	600MHz
+				SL8LM	733J		1.1GHz	600MHz
+				SL8A2	733J		1.1GHz	600MHz
+				SL89Z	753			1.2GHz	600MHz
+				SL8LL	753			1.2GHz	600MHz
+				SL8QG	778			1.6GHz	600MHz
+				SL89M	758			1.5GHz	600MHz
+				SL89R	723			1.0GHz	600MHz
+				SL8LT	733J		1.1GHz	600MHz
+				SL89Q	733J		1.1GHz	600MHz
+				SL89P	753			1.2GHz	600MHz
+				SL8LS	753			1.2GHz	600MHz
+				SL89N	738			1.4GHz	600MHz
+				SL89Y	738			1.4GHz	600MHz
+		     */
+			case 8:	nameptr+=sprintf (cpu->name, "%s", "(Dothan) [C-0]");
 					break;
 		}
 		break;
@@ -585,7 +632,14 @@ void Identify_Intel (struct cpudata *cpu)
 		cpu->errata_url = strdup (p4_errata);
 		switch (cpu->stepping) {
 		case 1:
-			//FIXME: Does this exist? Not in latest specupdate
+			//400FSB 256K L2
+			//SSpec MHz L3
+			//SL5G8 1.6 1M 
+			//SL5S4 1.6 1M
+			//SL5FZ 1.4 512K
+			//SL5RZ 1.4 512K
+			//SL5G2 1.5 512K
+			//SL5RW 1.5 512K
 			nameptr+=sprintf (nameptr, "%s", " [C0]");
 			break;
 		case 2:
@@ -621,6 +675,17 @@ void Identify_Intel (struct cpudata *cpu)
 				break;
 		}
 		switch (cpu->stepping) {
+		case 2:
+			//512K L2
+			//          L3
+			//SL6GZ 1.5 1M
+			//SL6KB 1.5 1M
+			//SL6H2 1.9 2M
+			//SL6KC 1.9 2M
+			//SL66Z 2.0 1M
+			//SL6KD 2.0 1M
+			nameptr+=sprintf (nameptr, "%s", " [A0]");
+			break;
 		case 4:
 			//SL66B 1.6GHz
 			//SL63X SL62P SL6BQ 1.8GHz
@@ -633,13 +698,29 @@ void Identify_Intel (struct cpudata *cpu)
 			nameptr+=sprintf (nameptr, "%s", " [B0]");
 			break;
 		case 5:
+			/*[M0] */
 			//SL6Z3 2.4GHz (800FSB)
 			//SL6Z5 2.8GHz (800FSB)
 			/* P4 Extreme edition.*/
 			//SL7AA 3.2GHz (800FSB) 2MB L3 cache
 			//SL7CH 3.4GHz (800FSB) 2MB L3 cache
+
+			/* 400FSB B1 512K L2 */
+			//SL6YJ 2.0 1M L3
+			//SL6Z6 2.0 1M L3
+			//SL6Z2 2.5 1M L3
+			//SL6Z7 2.5 1M L3
+			//SL6YL 2.8 2M L3
+			//SL6Z8 2.8 2M L3
 			nameptr+=sprintf (nameptr, "%s", " [M0]");
 			break;
+		case 6:
+			//400FSB 512K L2
+			//SL79V 3.0 4M L3
+			//SL79Z 2.7 2M L3
+			//SL7A5 2.2 2M L3
+			nameptr+=sprintf (nameptr, "%s", " [C0]");
+			break;
 		case 7:
 			//SL6HL SL6K6 2.8GHz (533MHz FSB)
 			//SL6LA SL6S6 1.8GHz
@@ -678,19 +759,156 @@ void Identify_Intel (struct cpudata *cpu)
 			break;
 		}
 		break;
-	case 0xF33:
-		// SL7D8 2.8GHz (533FSB)
-		// SL79K 2.8GHz (800FSB)
-		// SL79L 3.0GHz (800FSB)
-		// SL7B8 3.2GHz (800FSB)
-		// SL7B9 3.4GHz (800FSB)
-		// process = "0.09u";
-		// 125 million transistors
-		// 112mm2 die size
-		// pipeline_stages=31
-		nameptr+=sprintf (cpu->name, "%s", "Pentium 4 (Prescott) [C0]");
+	case 0xF30:
+		switch (cpu->stepping) {
+		case 3:
+			/*
+			sspec speed   fsb l2 90nm
+			SL7D7 2.26GHz 533 512K
+			SL7FY 2.4GHz  800 1M
+			SL7E8 2.4GHz  533 1M
+			SL7E9 2.66GHz 533 1M
+			SL7D8 2.8GHz  533 1M
+			SL79K 2.8GHz  800 1M
+			SL79L 3.0GHz  800 1M
+			SL79M 3.2GHz  800 1M
+			SL7B8 3.2GHz  800 1M
+			SL7B9 3.4GHz  800 1M
+			SL7AJ 3.4GHz  800 1M
+
+			process = "0.09u";
+			125 million transistors
+			112mm2 die size
+			pipeline_stages=31
+			*/
+			nameptr+=sprintf (cpu->name, "%s", "Pentium 4 (Prescott) [C0]");
+			break;
+		case 4:
+			/*
+			1M L2 90nm
+			sspec speed   fsb
+			SL7E2 2.8GHz  533
+			SL7E3 2.8GHz  800
+			SL7KA 2.8GHz  800
+			SL7K9 2.8GHz  533
+			SL7E4 3.0GHz  800
+			SL7KB 3.0GHz  800
+			SL7L4 3.0GHz  800
+			SL7L5 3.2GHz  800
+			SL7E5 3.2GHz  800
+			SL7KC 3.2GHz  800
+			SL7E6 3.4GHz  800
+			SL7KD 3.4GHz  800
+			SL7YP 2.4GHz  533
+			SL7YU 2.66GHz 533
+			SL7J4 2.8GHz  533
+			SL7J5 2.8GHz  800
+			SL7KH 2.8GHz  533
+			SL7KJ 2.8GHz  800
+			SL7YV 2.93GHz 533
+			SL7J6 3.0GHz  800
+			SL7KK 3.0GHz  800
+			SL7J7 3.2GHz  800
+			SL7KL 3.2GHz  800
+			SL7LA 3.2GHz  800
+			SL7J8 3.4GHz  800
+			SL7KM 3.4GHz  800
+			SL7L8 3.4GHz  800
+			SL7J9 3.6GHz  800
+			SL7KN 3.6GHz  800
+			SL7L9 3.6GHz  800
+			 */
+			nameptr+=sprintf (cpu->name, "%s", "Pentium 4 (Prescott) [D0]");
+			break;
+		}
 		break;
+
 	case 0xF40:
+		nameptr+=sprintf (cpu->name, "%s", "Pentium 4 ");
+		switch (cpu->stepping) {
+		case 1:
+			/*
+			 1M L2 90nm
+			SL88F 2.4GHz  533
+			SL8B3 2.66GHz 533
+			SL88G 2.8GHz  533
+			SL88H 2.8GHz  800
+			SL7PL 2.8GHz  800
+			SL7PK 2.8GHz  533
+			SL7PM 3GHz    800
+			SL88J 3GHz    800
+			SL7PN 3.2GHz  800
+			SL88K 3.2GHz  800
+			SL88L 3.4GHz  800
+			SL7PP 3.4GHz  800
+			SL7PT 2.66GHz 533
+			SL82P 2.8GHz  800
+			SL7PR 2.8GHz  800
+			SL8HX 2.8GHz  800
+			SL85U 2.66GHz 533
+			SL8J8 2.66GHz 533
+			SL85V 2.93GHz 533
+			SL8J9 2.93GHz 533
+			SL87L 3.06GHz 533
+			SL8JA 3.06GHz 533
+			SL82X 3.0GHz  800
+			SL7PU 3.0GHz  800
+			SL8HZ 3.0GHz  800
+			SL7PW 3.2GHz  800
+			SL7PX 3.2GHz  800
+			SL82Z 3.2GHz  800
+			SL8J2 3.2GHz  800
+			SL7PY 3.4GHz  800
+			SL7PZ 3.4GHz  800
+			SL833 3.4GHz  800
+			SL7ZW 3.4GHz  800
+			SL8J5 3.4GHz  800
+			SL84X 3.6GHz  800
+			SL7Q2 3.6GHz  800
+			SL7NZ 3.6GHz  800
+			SL8J6 3.6GHz  800
+			SL82U 3.8GHz  800
+			SL84Y 3.8GHz  800
+			SL7P2 3.8GHz  800
+			SL8J7 3.8GHz  800
+			 */
+			/*
+			 8MB L3 [C-0]
+			 SL8EY 3.3GHz 667
+			 SL8EW 3GHz   667
+			 4MB L3
+			 SL8ED 2.8GHz 667
+			 */
+			nameptr+=sprintf (cpu->name, "%s", "Pentium 4 (Prescott) [E0]");
+			break;
+		case 3:
+			/*
+			 2M L2 90nm
+			 SL7Z9 3.0GHz  800
+			 SL7Z8 3.2GHz  800
+			 SL8Z7 3.4GHz  800
+			 SL7Z5 3.6GHz  800
+			 SL7Z4 3.73GHz 800
+			 SL7Z3 3.8GHz  800
+			 */
+			nameptr+=sprintf (cpu->name, "%s", "Pentium 4 (Prescott) [N0]");
+			break;
+		case 4:
+			/*
+			   1Mx2 L2	800MHz FSB
+			   SL88T	2.8GHz
+			   SL88S	3GHz
+			   SL88R	3.2GHz
+			   SL8FK	3.2GHz
+			 */
+			nameptr+=sprintf (cpu->name, "%s", "Extreme Edition [A0]");
+			break;
+		default:
+			nameptr+=sprintf (cpu->name, "%s", "D (Foster)");
+			break;
+		}
+		break;
+
 	case 0xF50:
 		cpu->connector = CONN_SOCKET_603;
 //		cpu->datasheet_url = strdup (p4_478_datasheet);
