@@ -1,5 +1,5 @@
 /*
- *  $Id: cpuid.c,v 1.11 2002/09/02 15:57:36 davej Exp $
+ *  $Id: cpuid.c,v 1.12 2005/04/25 20:54:12 davej Exp $
  *	This file is part of x86info
  *	(C) 2000, 2001 Dave Jones.
  *	Fixes by Arjan van de Ven (arjanv@redhat.com) and
@@ -9,6 +9,8 @@
  *
  */
 
+#define _LARGEFILE64_SOURCE
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
@@ -18,7 +20,7 @@
 
 #include "x86info.h"
 
-void cpuid (int CPU_number, int idx,
+void cpuid (int CPU_number, unsigned int idx,
 	unsigned long *eax,
 	unsigned long *ebx,
 	unsigned long *ecx,
@@ -38,12 +40,12 @@ void cpuid (int CPU_number, int idx,
 	snprintf (cpuname,18, "/dev/cpu/%d/cpuid", CPU_number);
 	fh = open (cpuname, O_RDONLY);
 	if (fh != -1) {
-		lseek (fh, idx, SEEK_CUR);
+		lseek64 (fh, (off64_t)idx, SEEK_CUR);
 		read (fh, &buffer[0], 16);
-		if (eax!=0)	*eax = (*(unsigned long *)buffer);
-		if (ebx!=0)	*ebx = (*(unsigned long *)(buffer+4));
-		if (ecx!=0)	*ecx = (*(unsigned long *)(buffer+8));
-		if (edx!=0)	*edx = (*(unsigned long *)(buffer+12));
+		if (eax!=0)	*eax = (*(unsigned *)(buffer   ));
+		if (ebx!=0)	*ebx = (*(unsigned *)(buffer+ 4));
+		if (ecx!=0)	*ecx = (*(unsigned *)(buffer+ 8));
+		if (edx!=0)	*edx = (*(unsigned *)(buffer+12));
 		close (fh);
 	} else {
 		/* Something went wrong, just do UP and hope for the best. */
